/*
This code has been largely lifted from Claudio's twAjaxTools

Released under MIT license
*/
var AjaxFormManager = Class.create();

AjaxFormManager.prototype = {};
AjaxFormManager.prototype.initialize = function(id, options) {
    this.form = $(id);
    this.form_id = this.form.getAttribute('id');
    this.form.ajax = this;
    
    this.options = options || {};
    
    this.form.target = null;
    
    // Add an extra field containing form id to tell the ajax connector who to
    // call back.
    this.id_field = document.createElement('input');
    this.id_field.type = 'hidden';
    this.id_field.name = '_ajax_form_parent_id';
    this.id_field.value = this.form_id;
    this.form.appendChild(this.id_field);
    

    this.submit_button = $(this.form_id + '_submit');
    this.submit_button.name = "submit";
    
    this.form.observe("submit", this.submit_form.bindAsEventListener(this));
};


AjaxFormManager.prototype.submit_form = function(e)
{
    Event.stop(e);
    //create the spinner and insert into the page
    var spinner = document.createElement("div");
    spinner.id = this.form_id + "_spinner";
    spinner.innerHTML = "Saving .....";
    this.form.appendChild(spinner);

    //gather the form elements and their values
    var elements = this.form.elements;
    var tosend = {};
    for(var i=0; i<elements.length; i++)
    {
        tosend[elements[i].name] = elements[i].value;
    }

    //tosend = toJsonString(tosend);
    request_opt = {
                    method:'post',
                    parameters: tosend,
                    onComplete: this.submitted.bindAsEventListener(this),
                  }
    response = new Ajax.Request(this.form.action, request_opt);
}

AjaxFormManager.prototype.reset_form = function() {
    Form.reset(this.form);
    
    // Set the ID again because Form.reset clears the hidden field.
    this.id_field.value = this.form_id;
}

AjaxFormManager.prototype.submitted = function(response) {
    this.clear_form_errors();
    //remove the spinner 
    $(this.form.id + "_spinner").remove();
    this.submit_button.disabled = false;
    var failed = false;
    switch (response.statusText) { 
        case 'OK':
                if (this.options.clear_on_success) {
                    this.reset_form();
                }
                this.run_handler('on_success', response);
//                this.fire_event('on_success', response);
                break;
        case 'FAILED_VALIDATION':
                form_errors = JSON.parse(response.responseText).form_errors
                this.set_form_errors(form_errors);
                failed = true;
                this.run_handler('on_failure', response);
                break;
        case 'IDENTITY_FAILURE':
                if (!this.options.ignore_identity_flash && 
                     response.identity_flash) {
                    alert(response.identity_flash);
                }
                this.fire_event('on_identity_failure', response);
                failed = true;
                this.run_handler('on_failure', response);
                break;
        default:
                failed = true;
                alert('Error processing form: Status (' + 
                                response.status + ')');
    }
    
    if (failed) {
        this.fire_event('on_failure', response);
    }
    
    this.fire_event('on_complete', response);
    
    if (!this.options.ignore_flash && response.flash) {
        alert(response.flash);
    }
    
    if (!this.options.ignore_redirects && response.redirect) {
        window.location.href = response.redirect;
    }
};


AjaxFormManager.prototype.display_error = function (id, error) {
    if (typeof(error) != 'string') {
        $H(error).each(function(i) {
            this.display_error(id + '_' + i[0], i[1]);
        }.bind(this));
        return true;
    }
    
    var error_id = this.form_id + '_' + id + '_error';
    var error_container = $(error_id);
    
    if (error_container) {
        error_container.innerHTML = error;
        Element.show(error_container);
    } else {
        // if we can't find an error container we show it in an alert
        alert(this.form_id + '_' + id + ': ' + error);
    }
};


AjaxFormManager.prototype.set_form_errors = function (errors) {
    if (!errors) {
        errors = {};
    }
    
//    if (typeof(errors) == 'string') {
//        alert(errors);
//        return true;
//    }
    
    // display errors
    $H(errors).each(function(i) {
        this.display_error(i[0], i[1]);
    }.bind(this));
};


AjaxFormManager.prototype.clear_form_errors = function() {
    Form.getElements(this.form).each(function(element) {
        if (element.type == 'submit' || element.type == 'hidden') {
                return false;
        }
        
        var error_container = $(element.id + '_error');
        
        if (error_container) {
            error_container.innerHTML = '';
            Element.hide(error_container);
        }
    });
};

AjaxFormManager.prototype.run_handler = function(handler_name, data)
{
    var handler = window[this.form_id + "_" + handler_name + "_handler"];
    if(typeof(handler) != undefined)
    {
        handler(data);
    }
}


AjaxFormManager.prototype.fire_event = function(event, response) {
    if (this.options[event]) {
        _event = this.options[event];
        try {
            if (typeof(_event) == 'function') {
                _event(response);
            } else {
                eval(_event);
            }
        } catch (err) {
            alert('Error firing ' + event + ' (' + err + ')');
        }
    }
};


